<?php

namespace App\Util;

use App\Exception;

use PHPMailer\PHPMailer\PHPMailer;
use PHPMailer\PHPMailer\Exception as MailerException;

use App\Util\Security as Security;

class Util {
    
    private $connection;
    private $dbSchema = 'pt-br';
    
    public function __construct (\Doctrine\DBAL\Connection $connection) {
        $this->connection = $connection;
        //$this->loadDbSchema();
    }
    
    private function getDbSchema() {
        return $this->dbSchema;
    }
    
    private function loadDbSchema() {
        $configFile = file_get_contents(__DIR__ . "/../../environment/config.json");
        $config     = json_decode($configFile, true);
        
        if(isset($config["dbSchema"])){
            $this->dbSchema = $config["dbSchema"];
        }
    }
    
    public function removeDecimal($value, $offet = 2){
        return floatval(bcdiv($value,1,$offet));
    }
    
    public function subtract($a, $b){
        return floatval(bcsub($a,$b, 2));
    }
    
    public function add($a, $b){
        return floatval(bcadd($a,$b, 2));
    }
    
    public function mult($a, $b){
        return floatval(bcmul($a,$b, 2));
    }
    
    public function div($a, $b){
        return floatval(bcdiv($a,$b, 2));
    }
    
    public function newCode ($counterName, $size) {
        /* NOVOCODIGO */
        try {
            $nrseq = null;
            $connection = $this->connection;
            $storedProcedure = new StoredProcedure($connection, "NOVO_CODIGO_NT");
            $storedProcedure->addParam(new Param("P_CONTADOR", Param::PARAM_INPUT, $counterName, Param::PARAM_TYPE_STR, strlen($counterName)));
            $storedProcedure->addParam(new Param("P_SEQUENCIAL", Param::PARAM_OUTPUT, $nrseq, Param::PARAM_TYPE_INT, 20));
            $result = $storedProcedure->execute();
            return substr($result["P_SEQUENCIAL"], strlen($result["P_SEQUENCIAL"])-$size);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    /**
     *
     *
     * @param StoredProcedure $procedure The procedure to be executed.
     * @param array $paramValues Param values to override values present in Param instance.
     *
     * @return array The output parameters indexed by name.
     */
    public function executeProcedure(StoredProcedure $procedure, array $paramValues = array()) {
        $queryStmt = $this->getQueryString($procedure);
        $stmt = $this->connection->prepare($queryStmt);
        $valuesOutput = array();
        foreach ($procedure->getParams() as $param) {
            if ($param->isOutput()) {
                $valuesOutput[$param->getName()] = $param->getValue();
                $stmt->bindParam($param->getParamAlias(), $valuesOutput[$param->getName()], $param->getType(), $param->getLength());
            } else {
                $stmt->bindValue($param->getParamAlias(), $param->getValue(), $param->getType());
            }
        }

        $stmt->execute();
        return $valuesOutput;
    }

    /**
     * @param StoredProcedure $procedure
     * @return string
     */
    protected function getQueryString(StoredProcedure $procedure) {
        $paramNames = array_map(function (Param $param) {
            return ":" . $param->getName();
        }, $procedure->getParams());
        $paramString = implode(", ", $paramNames);
        return "CALL " . $procedure->getName() . " ( " . $paramString . " )";
    }
    
    public function validadeRequestBody ($body) {
        $requiredBasicFields    = ["date", "saleId", "subsidiaryCode", "orderType", "orderNumber", "subTotal", "total"];
        $requiredconsumerFields = ["clientId", "name"];
        $requiredAddressFields  = ["street", "uf", "zipCode", "number", "city", "neighborhood"];
        $requiredPaymentFields  = ["paymentId", "value"];
        $basicInfor             = $body;
        $consumerinfo           = $body["consumerDetail"];
        $addressinfo            = $consumerinfo ["address"];
        $paymentinfo            = $body ["payments"];
        
        if($body['orderType'] == 1) { //delivery type
            foreach($requiredBasicFields as $basicInf) {
                if (empty($basicInfor[$basicInf])) {
                    throw new Exception\HTTPBodyException("O campo $basicInf é obrigatorio!");
                }
            }
    
            foreach($paymentinfo as $payment) {
                foreach ($requiredPaymentFields as $fieldPay) {
                    if (empty($payment[$fieldPay])) {
                        throw new Exception\HTTPBodyException("O campo $fieldPay é obrigatorio!"); 
                    }
                }
            }
        
            foreach ($requiredAddressFields as $address) {
                if (!isset($addressinfo[$address])) {
                    throw new Exception\HTTPBodyException("O campo $address é obrigatorio!"); 
                }
            }
            
            foreach ($requiredconsumerFields as $dateConsu) {
                if (empty($consumerinfo[$dateConsu])) {
                    throw new Exception\HTTPBodyException("O campo $dateConsu é obrigatorio!"); 
                }
            }
        }
        
        $basicArr = array_filter($requiredBasicFields, function($key) use ($body) {
            return !isset($body[$key]) ? true : empty($body[$key]) ? true : false;
        });
        
        $consumerArr = array_filter($requiredconsumerFields, function($key) use ($body) {
            return !isset($body["consumerDetail"][$key]) ? true : empty($body["consumerDetail"][$key]) ? true : false;
        });
        
        $paymentArr = [];
        if (count($body["payments"]) >= 1) {
            foreach ($body["payments"] as $payment) {
                $paymentArr = array_filter($requiredPaymentFields, function($key) use ($payment) {
                    return !isset($payment[$key]) ? true : empty($payment[$key]) ? true : false;
                });
            }
        } else throw new Exception\HTTPBodyException("Nenhum método de pagamento enviado.");
        
        if (empty($body["consumerDetail"]["email"]) && empty($body["consumerDetail"]["phone"]) && empty($body["consumerDetail"]["cpf"]))
            throw new Exception\HTTPBodyException("Um dos campos para identificação de consumidor deve ser enviado: 'email', 'phone' ou 'cpf'.");
        
        $allFields = array_merge($basicArr, $consumerArr, $paymentArr);
        if (count($allFields) >= 1) {
            $fields = join(", ", $allFields);
            throw new Exception\HTTPBodyException("O(s) campo(s): $fields; Não foi(ram) enviado(s) ou tem valor(es) vazio(s).");
        }
        
        if (count($body["ItemOrder"]) == 0 && count($body["CombinedProducts"]) == 0)
            throw new Exception\HTTPBodyException("O pedido foi enviado sem itens.");
    }

    public function sendMail ($recipient, $cc, $subject, $body, $params) {
        global $CLIENT_DATA;
        
        try {
            $config = $CLIENT_DATA;
            
            $mail = new PHPMailer(true);
            
            $mail->isSMTP(); // Set mailer to use SMTP
            $mail->Host       = $config["host"]; // Specify main and backup SMTP servers
            $mail->SMTPAuth   = !empty($config["SMTPSecure"]); // Enable SMTP authentication
            $mail->Username   = $config["username"]; // SMTP username
            $mail->Password   = !empty($config["password"]) ? Security::decrypt($config["password"]):""; // SMTP password
            $mail->SMTPSecure = !empty($config["SMTPSecure"]) ? $config["SMTPSecure"]:false; // Enable TLS encryption, `ssl` also accepted
            $mail->Port       = $config["port"]; // TCP port to connect to
            $mail->CharSet    = "UTF-8";
            $mail->Encoding   = "base64";


            $mail->setFrom($config["from"], "Teknisa Varejo");
            foreach ($recipient as $r) { $mail->addAddress($r); }
            foreach ($cc as $c) { $mail->addCC($c); }
            
            $mail->isHTML(true);                                  // Set email format to HTML
            $mail->Subject = "$subject - " . $config["client"];
            $mail->Body    = str_replace(array_keys($params), array_values($params), $body);
            
            $mail->send();

        } catch (\Exception $e) {
            var_dump("error: ".$e->getMessage());
        }
    }

    public function formatNrSeqProdCom($seq) {
        return str_pad((string) ((int) $seq), 3, '0', STR_PAD_LEFT);
    }

    public function writeLog($type, $fileName, $content) {
        global $CLIENT_DATA;
        $config = $CLIENT_DATA;

        $clientFolder = $this->connection->getParams();
        $clientFolder = (isset($clientFolder['dbname'])) ? $clientFolder['dbname'] : "undefined";
        $basePath     = $config['logPath'] . "/$clientFolder";
        
        if($type == 'order') {
            $folder      = $basePath . "/orders/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }

        if($type == 'order_processed') {
            $folder      = $basePath . "/processed/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }

        if($type == 'order_error') {
            $folder      = $basePath . "/error/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";

            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, json_encode($content, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
        }

        if($type == 'network') {
            $folder      = $basePath . "/networkError/" . date("Y") . "/" . date("m") . "/" . date("d") . "/";
            $currentFile = "$folder/$fileName";

            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }
            
            file_put_contents($currentFile, $content);
        }
        
        if($type == 'salesOrigin') {
            $folder      = $basePath . "/salesOrigin/" . date("Y") . "/" . date("m") . "/" . date("d");
            $currentFile = "$folder/$fileName";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, $content, FILE_APPEND);
        }
        
        if($type == 'execution') {
            $folder      = $basePath . "/execution/" . date("Y") . "/" . date("m") . "/";
            $currentFile = "$folder" . date("d") . ".txt";
            if(!file_exists($folder)) {
                mkdir($folder, 0775, true);
            }

            file_put_contents($currentFile, $content, FILE_APPEND);
        }

    }
    
    public function formatNetworkError($networkData) {
        $eventTime = $networkData['eventTime'];
        $header    = implode($networkData['header'], "\r\n");
        $url       = $networkData['url'];
        $method    = $networkData['method'];
        $params    = is_array($networkData['params']) ? implode($networkData['params'], "\r\n") : $networkData['params'];
        $httpCode  = $networkData['httpCode'];
        $error     = $networkData['error'];
        $response  = $networkData['response'];

        $template = file_get_contents(__DIR__ . '/../Templates/networkError.txt');
        $templateParams = [
            ':EVENT'     => $eventTime,
            ':HEADER'    => $header,
            ':URL'       => $url,
            ':METHOD'    => $method,
            ':PARAMS'    => $params,
            ':HTTP_CODE' => $httpCode,
            ':ERROR'     => $error,
            ':RESPONSE'  => $response            
        ];

        return str_replace(array_keys($templateParams), array_values($templateParams), $template);
    }   
    
    public function tofloat($num) {
        $dotPos = strrpos($num, '.');
        $commaPos = strrpos($num, ',');
        $sep = (($dotPos > $commaPos) && $dotPos) ? $dotPos : 
            ((($commaPos > $dotPos) && $commaPos) ? $commaPos : false);
       
        if (!$sep) {
            return floatval(preg_replace("/[^0-9]/", "", $num));
        } 
    
        $tmp = floatval(
            preg_replace("/[^0-9]/", "", substr($num, 0, $sep)) . '.' .
            preg_replace("/[^0-9]/", "", substr($num, $sep+1, strlen($num)))
        );
        
        $tmp2 = str_replace(".", ",", $tmp);

        if($this->getDbSchema() == 'en-us') {
            $tmp2 = str_replace(",", ".", $tmp2);
        }
        
        return $tmp2; 
    }
    
    public function to_Float($num) {
        $num = str_replace(",", ".", $num);
        $floatNumber = bcadd($num, 0, 2);
        return floatval($floatNumber);
    }

    
    public function getMailList($origin) {
        $list = [];
        
        $origin = preg_replace("/[^0-9A-Z]/", "", $origin);
        $origin = substr($origin, 0, 5);
        
        $params = [
            ':IDSUBPROJETO' => $origin
        ];
        
        $sqlSubProj = "
            SELECT CDSUBPROJETO 
              FROM CADASTROSUBPROJETO
             WHERE IDSUBPROJETO = :IDSUBPROJETO
        ";
        
        $result = $this->connection->fetchAssoc($sqlSubProj, $params);
        
        if(empty($result)) {
            throw new Exception\GeneralErrorException("Origem do pedido inválida.");
        }
        
        $subProj = $result['CDSUBPROJETO'];
  
        $params = [
            ':CDSUBPROJETO' => $subProj
        ];

        $sqlMailList = "
            SELECT DISTINCT DSEMAILTEC
              FROM LISTAEMAILTEC
             WHERE (CDSUBPROJETO = :CDSUBPROJETO OR CDSUBPROJETO IS NULL)
               AND (CDPROJETO = '014' OR CDPROJETO = '007')
            ORDER BY DSEMAILTEC
        ";
        
        $result = $this->connection->fetchAll($sqlMailList, $params);
       
       
        if(empty($result)) {
            throw new Exception\GeneralErrorException("Cadastro de responsáveis técnicos inexistente.");
        }
        
        foreach($result as $row) {
            if(!empty($row['DSEMAILTEC']) && filter_var($row['DSEMAILTEC'], FILTER_VALIDATE_EMAIL)) {
                array_push($list, $row['DSEMAILTEC']);
            }
        }
        
        if(empty($list)) {
            throw new Exception\GeneralErrorException("Cadastro de responsáveis técnicos inexistente.");
        }
        
        return $list;
    }
    
    public function getCurrentTimezone($state) {
        $timezones = [
            "AC" => "America/Rio_branco",   "AL" => "America/Maceio",
            "AP" => "America/Belem",        "AM" => "America/Manaus",
            "BA" => "America/Bahia",        "CE" => "America/Fortaleza",
            "DF" => "America/Fortaleza",    "ES" => "America/Fortaleza",
            "GO" => "America/Fortaleza",    "MA" => "America/Fortaleza",
            "MT" => "America/Cuiaba",       "MS" => "America/Campo_Grande",
            "MG" => "America/Fortaleza",    "PR" => "America/Fortaleza",
            "PB" => "America/Fortaleza",    "PA" => "America/Belem",
            "PE" => "America/Recife",       "PI" => "America/Fortaleza",
            "RJ" => "America/Fortaleza",    "RN" => "America/Fortaleza",
            "RS" => "America/Fortaleza",    "RO" => "America/Porto_Velho",
            "RR" => "America/Boa_Vista",    "SC" => "America/Fortaleza",
            "SE" => "America/Maceio",       "SP" => "America/Fortaleza",
            "TO" => "America/Araguaia",    
        ];
       
        return $timezones[$state];
    }
    
    public function getCurrentDateTime() {
    	return (new \DateTime())->format('Y-m-d\TH:i:s:uO');
    }    
    
    public function isValidCPF($cpf) {
       $cpf = preg_replace( '/[^0-9]/is', '', $cpf );
        
       if (strlen($cpf) == 11) {
           
            // Extrai somente os números
            $cpf = preg_replace( '/[^0-9]/is', '', $cpf );
             
            // Verifica se foi informado todos os digitos corretamente
            if (strlen($cpf) != 11) {
                return false;
            }
        
            // Verifica se foi informada uma sequência de digitos repetidos. Ex: 111.111.111-11
            if (preg_match('/(\d)\1{10}/', $cpf)) {
                return false;
            }
        
            // Faz o calculo para validar o CPF
            for ($t = 9; $t < 11; $t++) {
                for ($d = 0, $c = 0; $c < $t; $c++) {
                    $d += $cpf[$c] * (($t + 1) - $c);
                }
                $d = ((10 * $d) % 11) % 10;
                if ($cpf[$c] != $d) {
                    return false;
                }
            }
               
           }
       if (strlen($cpf) == 14) {
       
            $cpf = preg_replace('/[^0-9]/', '', (string) $cpf);
    	
        	// Valida tamanho
        	if (strlen($cpf) != 14)
        		return false;
    
        	// Verifica se todos os digitos são iguais
        	if (preg_match('/(\d)\1{13}/', $cpf))
        		return false;	
        
        	// Valida primeiro dígito verificador
        	for ($i = 0, $j = 5, $soma = 0; $i < 12; $i++)
        	{
        		$soma += $cpf[$i] * $j;
        		$j = ($j == 2) ? 9 : $j - 1;
        	}
        
        	$resto = $soma % 11;
        
        	if ($cpf[12] != ($resto < 2 ? 0 : 11 - $resto))
        		return false;
        
        	// Valida segundo dígito verificador
        	for ($i = 0, $j = 6, $soma = 0; $i < 13; $i++)
        	{
            		$soma += $cpf[$i] * $j;
            		$j = ($j == 2) ? 9 : $j - 1;
        	}
        
        	$resto = $soma % 11;
        
        	return $cpf[13] == ($resto < 2 ? 0 : 11 - $resto);
        } 
       

        return true;
    }
    
    public function removeAcentos($texto) {
        return preg_replace(array("/(á|à|ã|â|ä)/","/(Á|À|Ã|Â|Ä)/","/(é|è|ê|ë)/","/(É|È|Ê|Ë)/","/(í|ì|î|ï)/","/(Í|Ì|Î|Ï)/","/(ó|ò|õ|ô|ö)/","/(Ó|Ò|Õ|Ô|Ö)/","/(ú|ù|û|ü)/","/(Ú|Ù|Û|Ü)/","/(ñ)/","/(Ñ)/","/(ç)/","/(Ç)/"),explode(" ","a A e E i I o O u U n N c C"),$texto);
    }
    
}
